jQuery.expr[':'].icontains = function(a, i, m) {
    return jQuery(a).text().toUpperCase()
        .indexOf(m[3].toUpperCase()) >= 0;
};

$(document).ready(() => {

    // Enable all tooltips on the page
    const tooltipTriggerList = document.querySelectorAll('[data-bs-toggle="tooltip"]')
    const tooltipList = [...tooltipTriggerList].map(tooltipTriggerEl => new bootstrap.Tooltip(tooltipTriggerEl))

    teamsLoading(true);  // show the loading icon
    fetchAndLoadTeams(); // load the teams

    var searchTimeout = null;

    $("#search").keyup(() => {
        clearTimeout(searchTimeout);
        teamsLoading(true);

        searchTimeout = setTimeout(() => {
            fetchAndLoadTeams($("#search").val());
        }, 500)
    });

    $("#searchButton").on("click", () => {
        fetchAndLoadTeams($("#search").val());
    });
})

/**
 * Returns bool if the string is empty or only contains whitespace
 * 
 * **/
function isEmptyOrSpaces(string) {
    return string === null || string.match(/^ *$/) !== null;
}

function loadTeams(teams, highlightText="") {
    $("#teamsContainer").html("");  // Clear the previous listed teams

    if (teams.length < 1) {
        $("#teamsNotFound").show();
    }

    // Iterate over and add each team
    teams.forEach((team) => {
        $("#teamsContainer").append(
            `<div class='col-12 col-md-6 col-xl-4 mb-4'>
                <div
                    class='team p-4 bg-body-tertiary rounded h-100 fluid-hover-zoom shadow-sm shadow-on-hover'
                    data-number='${team.team_number}'>
                    <h4>
                        <span class='fs-6'>TEAM</span> 
                        ${team.team_number}
                        <span class='fs-5 text-secondary'>
                            <span class='fs-6'>SECTION</span>
                            ${team.section_letter}
                        </span>
                    </h4>
                    <ul class='list-unstyled team-members mt-3'>
                    </ul>
                </div>
            </div>`
        );

        // Whilimage.pnge we have the team, iterate over and add it's members
        team.members.forEach((member) => {
            const fullname = member.first + " " + member.last;

            $("#teamsContainer").find(".team-members").last().append(
                `<li class='bg-body-secondary mb-3 rounded w-100'>
                    <div 
                        type='button' 
                        class='team-member px-3 py-2 d-flex'
                        data-first='${member.first}'
                        data-last='${member.last}'
                        data-member-id='${member.id}'
                        data-team-number='${member.team}'
                        data-peg-number='${member.peg}'>
                        <span class='text-secondary me-2'>
                            ${member.peg}
                        </span>
                        <span class='team-member-fullname'>
                            ${fullname}
                        </span>
                        <span class='ms-auto text-secondary'>
                            <i class='bi bi-pencil'></i>
                        </span>
                    </div>
                </li>`
            );
        });
    });

    // Highlight all instances where the text matches the search critera
    if (!isEmptyOrSpaces(highlightText)) {
        $("#teamsContainer").find('.team-member-fullname').each(function() {
            var regex = new RegExp(highlightText, 'gi');
            $(this).html($(this).text().replace(regex, '<span class="text-primary fw-bold">$&</span>'));
        });
    }

    $(".team-member").on("click", function() {
        openEditModal($(this).data("member-id"));
    });
}

function fetchAndLoadTeams(search="") {
    $.ajax({
        url: getTeamsUrl,
        type: "post",
        data: {
            "csrfmiddlewaretoken": csrfMiddlewareToken,
            "search": !isEmptyOrSpaces(search) ? search : null
        },
        error: (xhr, textStatus, errorThrown) => { alert(errorThrown); },
        success: (result) => {
            teamsLoading(false);
            loadTeams(result.teams, search);
        }
    });
}

function teamsLoading(show=true) {
    $("#teamsNotFound").hide()

    if (show) {
        $("#teamsContainer").hide();
        $("#teamsLoadingSpinner").show();
        return
    }

    $("#teamsContainer").show();
    $("#teamsLoadingSpinner").hide();
}

function openEditModal(memberId) {

    // Member data
    const member = $(`.team-member[data-member-id='${memberId}']`);
    const first = member.data("first");
    const last = member.data("last");
    const teamNumber = member.data("team-number");
    const pegNumber = member.data("peg-number");

    // Load teams as options
    $("#editMemberTeam").html("");
    $(".team").map(function() {
        return $(this).data("number");
    }).get().forEach((team) => {
        $("#editMemberTeam").append($(`<option value='${team}'>Team ${team}</option>`));
    });

    // Load data to form
    $("#editMemberName").text(first + " " + last);
    $("#editMemberFirstName").val(first);
    $("#editMemberLastName").val(last);
    $("#editMemberTeam").val(teamNumber);
    $("#editMemberPeg").val(pegNumber);

    $("#editMemberModal").modal("show");

    // Update the submit button
    $("#saveEditModal").off("click").on("click", () => { saveEditModal(memberId); });
}

function saveEditModal(memberId) {

    // Grab the updated data
    const first = $("#editMemberFirstName").val();
    const last = $("#editMemberLastName").val();
    const teamNumber = $("#editMemberTeam").val();
    const pegNumber = $("#editMemberPeg").val();
    const search = $("#search").val();

    teamsLoading(true);

    $.ajax({
        url: updateMemberUrl,
        type: "post",
        data: {
            "csrfmiddlewaretoken": csrfMiddlewareToken,
            "memberId": memberId,
            "first": first,
            "last": last,
            "teamNumber": teamNumber,
            "pegNumber": pegNumber,
            "search": !isEmptyOrSpaces(search) ? search : null
        },
        error: (xhr, textStatus, errorThrown) => { alert(errorThrown); },
        success: (result) => {
            teamsLoading(false);
            loadTeams(result.teams, search);
            $("#editMemberModal").modal("hide");
        }
    });
}